
class StandardArg:
    def __init__(self, line: str):
        self.flag = ""
        self.value = ""
        self.position = ""

class CollectionOfArgs:
    def __init__(self, std):
        pass

    def build_collection(self):
        pass
